/*
 
Copyright (C) 2006 NTT DATA Corporation
 
This program is free software; you can redistribute it and/or
Modify it under the terms of the GNU General Public License 
as published by the Free Software Foundation, version 2.
 
This program is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied 
warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
PURPOSE.  See the GNU General Public License for more details.
 
*/

package com.clustercontrol.repository.dao;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;

import javax.ejb.CreateException;
import javax.ejb.DuplicateKeyException;
import javax.ejb.EJBException;
import javax.ejb.FinderException;
import javax.ejb.RemoveException;
import javax.naming.CommunicationException;
import javax.naming.NameAlreadyBoundException;
import javax.naming.NamingException;
import javax.naming.directory.Attributes;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.clustercontrol.accesscontrol.util.LdapConnectionManager;
import com.clustercontrol.bean.FacilityConstant;
import com.clustercontrol.bean.SnmpVersionConstant;
import com.clustercontrol.repository.bean.FacilityAttributeConstant;
import com.clustercontrol.repository.ejb.entity.FacilityBean;
import com.clustercontrol.repository.ejb.entity.FacilityPK;
import com.clustercontrol.util.apllog.AplLogger;

/**
 * Facility(BMP Entity Bean)のDAO実装クラス
 *
 * @version 1.0.0
 * @since 1.0.0
 */
public class FacilityDAOImpl implements FacilityDAO {
	protected static Log m_log = LogFactory.getLog( FacilityDAOImpl.class );
	
	protected static final String OBJECTCLASS = "objectClass";
	protected static final String BASE_DN = "ou=facility";
	protected static final String OBJECT_CLASS_NODE = "ccNode";
	protected static final String OBJECT_CLASS_SCOPE = "ccScope";
	protected static final String OBJECT_CLASS_FACILITY = "ccFacility";
	/** LDAP用コンテキスト取得時のリトライ上限 */
	private static int MAX_RETRY = 3;
	
	/* (non-Javadoc)
	 * @see com.clustercontrol.repository.dao.FacilityDAO#init()
	 */
	public void init() {
		
	}

	/* (non-Javadoc)
	 * @see com.clustercontrol.repository.dao.FacilityDAO#load(com.clustercontrol.repository.ejb.FacilityPK, com.clustercontrol.repository.ejb.FacilityBean)
	 */
	public void load(FacilityPK pk, FacilityBean ejb) throws EJBException {
		javax.naming.directory.DirContext ctx = null;
		
		int retryCount=0;
		while(MAX_RETRY > retryCount){
				
			try {
				clearBean(ejb);
				
				// LDAP用のコンテキストの作成
				ctx = LdapConnectionManager.getConnectionManager().getDirContext();
				
				//dnをキーとしてLDAPアクセスし、属性を取得
//				javax.naming.directory.Attributes attrs = ctx.getAttributes(pk.getDn());
				javax.naming.directory.Attributes attrs = ctx.getAttributes(createDnString(pk.getFacilityId()));
				
				//facilityIdからdnの文字列を作成する
//				ejb.setDn(pk.getDn());
				ejb.setDn(createDnString(pk.getFacilityId()));
				
				//ccFacilityId取得
//				ejb.setFacilityId((String)attrs.get(FacilityAttributeConstant.FACILITYID).get(0));
				ejb.setFacilityId(pk.getFacilityId());
				//cn取得
				ejb.setCn((String)attrs.get(FacilityAttributeConstant.CN).get(0));
				//description取得
				if(attrs.get(FacilityAttributeConstant.DESCRIPTION) != null){
					ejb.setDescription((String)attrs.get(FacilityAttributeConstant.DESCRIPTION).get(0));
				}
				//objectClassを取得し、ノードかスコープの判断をする
				javax.naming.NamingEnumeration objectClass = attrs.get(OBJECTCLASS).getAll();
				while ( objectClass.hasMoreElements() ) {
					String name = (String)objectClass.nextElement();
					if(name.compareTo(OBJECT_CLASS_NODE) == 0){
						ejb.setType(FacilityConstant.TYPE_NODE);
						break;
					}
					else if(name.compareTo(OBJECT_CLASS_SCOPE) == 0){
						ejb.setType(FacilityConstant.TYPE_SCOPE);
						break;
					}
				}
				
				if(ejb.getType() == FacilityConstant.TYPE_NODE){
					//ccAgentControl取得
					if(attrs.get(FacilityAttributeConstant.AGENTCONTROL) != null){
						ejb.setAgentControl(stringToBoolean((String)attrs.get(FacilityAttributeConstant.AGENTCONTROL).get(0)));
					}
					//ccAgentWatch取得
					if(attrs.get(FacilityAttributeConstant.AGENTWATCH) != null){
						ejb.setAgentWatch(stringToBoolean((String)attrs.get(FacilityAttributeConstant.AGENTWATCH).get(0)));
					}
					//ccAgentWatchStatus取得
					if(attrs.get(FacilityAttributeConstant.AGENTWATCHSTATUS) != null){
						ejb.setAgentWatchStatus(stringToBoolean((String)attrs.get(FacilityAttributeConstant.AGENTWATCHSTATUS).get(0)));
					}
					//ccCharSet取得
					if(attrs.get(FacilityAttributeConstant.CHARSET) != null){
						ejb.setCharSet((String)attrs.get(FacilityAttributeConstant.CHARSET).get(0));
					}
					//ccCreateTimestamp取得
					if(attrs.get(FacilityAttributeConstant.CREATETIMESTAMP) != null){
						ejb.setCreateTimestamp(stringToDate((String)attrs.get(FacilityAttributeConstant.CREATETIMESTAMP).get(0)));
					}
					//ccCreatorsName取得
					if(attrs.get(FacilityAttributeConstant.CREATORSNAME) != null){
						ejb.setCreatorsName((String)attrs.get(FacilityAttributeConstant.CREATORSNAME).get(0));
					}
					//ccValid取得
					if(attrs.get(FacilityAttributeConstant.VALID) != null){
						ejb.setValid(stringToBoolean((String)attrs.get(FacilityAttributeConstant.VALID).get(0)));
					}
					//ccDhcpClient取得
					if(attrs.get(FacilityAttributeConstant.DHCPCLIENT) != null){
						ejb.setDhcpClient(stringToBoolean((String)attrs.get(FacilityAttributeConstant.DHCPCLIENT).get(0)));
					}
					//host取得
					if(attrs.get(FacilityAttributeConstant.HOST) != null){
						javax.naming.NamingEnumeration hosts = attrs.get(FacilityAttributeConstant.HOST).getAll();
						ArrayList hostList = new ArrayList();
						while ( hosts.hasMoreElements() ) {
							String host = (String)hosts.nextElement();
							hostList.add(host);
						}
						ejb.setHost(hostList);
					}
					//ipNetworkNumber取得
					if(attrs.get(FacilityAttributeConstant.IPNETWORKNUMBER) != null){
						ejb.setIpNetworkNumber((String)attrs.get(FacilityAttributeConstant.IPNETWORKNUMBER).get(0));
					}
					//ccIpNetworkNumberV6取得
					if(attrs.get(FacilityAttributeConstant.IPNETWORKNUMBERV6) != null){
						ejb.setIpNetworkNumberV6((String)attrs.get(FacilityAttributeConstant.IPNETWORKNUMBERV6).get(0));
					}
					//ipProtocolNumber取得
					if(attrs.get(FacilityAttributeConstant.IPPROTOCOLNUMBER) != null){
						ejb.setIpProtocolNumber(new Integer((String)attrs.get(FacilityAttributeConstant.IPPROTOCOLNUMBER).get(0)));
					}
					//ccIpType取得
					if(attrs.get(FacilityAttributeConstant.IPTYPE) != null){
						ejb.setIpType(new Integer((String)attrs.get(FacilityAttributeConstant.IPTYPE).get(0)));
					}
					//ccLogWatch取得
					if(attrs.get(FacilityAttributeConstant.LOGWATCH) != null){
						ejb.setLogWatch(stringToBoolean((String)attrs.get(FacilityAttributeConstant.LOGWATCH).get(0)));
					}
					//ccMachine取得
					if(attrs.get(FacilityAttributeConstant.MACHINE) != null){
						ejb.setMachine((String)attrs.get(FacilityAttributeConstant.MACHINE).get(0));
					}
					//ccManagerContact取得
					if(attrs.get(FacilityAttributeConstant.MANAGERCONTACT) != null){
						ejb.setManagerContact((String)attrs.get(FacilityAttributeConstant.MANAGERCONTACT).get(0));
					}
					//ccManagerName取得
					if(attrs.get(FacilityAttributeConstant.MANAGERNAME) != null){
						ejb.setManagerName((String)attrs.get(FacilityAttributeConstant.MANAGERNAME).get(0));
					}
					//ccModifiersName取得
					if(attrs.get(FacilityAttributeConstant.MODIFIERSNAME) != null){
						ejb.setModifiersName((String)attrs.get(FacilityAttributeConstant.MODIFIERSNAME).get(0));
					}
					//ccModifyTimestamp取得
					if(attrs.get(FacilityAttributeConstant.MODIFYTIMESTAMP) != null){
						ejb.setModifyTimestamp(stringToDate((String)attrs.get(FacilityAttributeConstant.MODIFYTIMESTAMP).get(0)));
					}
					//ccNodeName取得
					if(attrs.get(FacilityAttributeConstant.NODENAME) != null){
						ejb.setNodeName((String)attrs.get(FacilityAttributeConstant.NODENAME).get(0));
					}
					//ccOsName取得
					if(attrs.get(FacilityAttributeConstant.OSNAME) != null){
						ejb.setOsName((String)attrs.get(FacilityAttributeConstant.OSNAME).get(0));
					}
					//ccOsRelease取得
					if(attrs.get(FacilityAttributeConstant.OSRELEASE) != null){
						ejb.setOsRelease((String)attrs.get(FacilityAttributeConstant.OSRELEASE).get(0));
					}
					//ccOsVersion取得
					if(attrs.get(FacilityAttributeConstant.OSVERSION) != null){
						ejb.setOsVersion((String)attrs.get(FacilityAttributeConstant.OSVERSION).get(0));
					}
					//ccPingWatch取得
					if(attrs.get(FacilityAttributeConstant.PINGWATCH) != null){
						ejb.setPingWatch(stringToBoolean((String)attrs.get(FacilityAttributeConstant.PINGWATCH).get(0)));
					}
					//ccPingWatchStatus取得
					if(attrs.get(FacilityAttributeConstant.PINGWATCHSTATUS) != null){
						ejb.setPingWatchStatus(stringToBoolean((String)attrs.get(FacilityAttributeConstant.PINGWATCHSTATUS).get(0)));
					}
					//ccPlatform取得
					if(attrs.get(FacilityAttributeConstant.PLATFORM) != null){
						ejb.setPlatform((String)attrs.get(FacilityAttributeConstant.PLATFORM).get(0));
					}
					//ccStatusWatch取得
					if(attrs.get(FacilityAttributeConstant.STATUSWATCH) != null){
						ejb.setStatusWatch(stringToBoolean((String)attrs.get(FacilityAttributeConstant.STATUSWATCH).get(0)));
					}
					//ccSnmpCommunity取得
					if(attrs.get(FacilityAttributeConstant.SNMPCOMMUNITY) != null){
						ejb.setSnmpCommunity((String)attrs.get(FacilityAttributeConstant.SNMPCOMMUNITY).get(0));
					}
					//ccSnmpPort取得
					if(attrs.get(FacilityAttributeConstant.SNMPPORT) != null){
						ejb.setSnmpPort(new Integer((String)attrs.get(FacilityAttributeConstant.SNMPPORT).get(0)));
					}
					//ccSnmpVersion取得
					if(attrs.get(FacilityAttributeConstant.SNMPVERSION) != null){
						ejb.setSnmpVersion((String)attrs.get(FacilityAttributeConstant.SNMPVERSION).get(0));
					}
					
					//ccFacilitySortValue取得
					if(attrs.get(FacilityAttributeConstant.SORT_VALUE) != null){
						ejb.setSnmpVersion((String)attrs.get(FacilityAttributeConstant.SORT_VALUE).get(0));
					}

					
					//ccSnmpTimeout取得
					if(attrs.get(FacilityAttributeConstant.SNMPTIMEOUT) != null){
						ejb.setSnmpTimeout(new Integer((String)attrs.get(FacilityAttributeConstant.SNMPTIMEOUT).get(0)));
					}

					//ccSnmpRetries取得
					if(attrs.get(FacilityAttributeConstant.SNMPRETRIES) != null){
						ejb.setSnmpRetries(new Integer((String)attrs.get(FacilityAttributeConstant.SNMPRETRIES).get(0)));
					}
					
					//ccSnmpProxy取得
					if(attrs.get(FacilityAttributeConstant.SNMPPROXY) != null){
						ejb.setSnmpProxy((String)attrs.get(FacilityAttributeConstant.SNMPPROXY).get(0));
					}

					//ccNote取得
					if(attrs.get(FacilityAttributeConstant.NOTE) != null){
						javax.naming.NamingEnumeration notes = attrs.get(FacilityAttributeConstant.NOTE).getAll();
						ArrayList noteList = new ArrayList();
						while ( notes.hasMoreElements() ) {
							String note = (String)notes.nextElement();
							noteList.add(note);
						}
						ejb.setNote(noteList);
					}
					
					//ccIconImage取得
					if(attrs.get(FacilityAttributeConstant.ICONIMAGE) != null){
						ejb.setIconImage((String)attrs.get(FacilityAttributeConstant.ICONIMAGE).get(0));
					}

					//ccVirtualizationNodeType取得
					if(attrs.get(FacilityAttributeConstant.VIRTNODETYPE) != null){
						ejb.setVirtNodeType((String)attrs.get(FacilityAttributeConstant.VIRTNODETYPE).get(0));
					}
					
					//ccVMManagementNode取得
					if(attrs.get(FacilityAttributeConstant.VMMANAGEMENTNODE) != null){
						ejb.setVMManagementNode((String)attrs.get(FacilityAttributeConstant.VMMANAGEMENTNODE).get(0));
					}
					
					//ccVMINDEX取得
					if(attrs.get(FacilityAttributeConstant.VMINDEX) != null){
						ejb.setVmIndex(new Integer((String)attrs.get(FacilityAttributeConstant.VMINDEX).get(0)));
					}
					
					//ccVMName取得
					if(attrs.get(FacilityAttributeConstant.VMNAME) != null){
						ejb.setVmName((String)attrs.get(FacilityAttributeConstant.VMNAME).get(0));
					}
					
					//ccVirtualizationSolution取得
					if(attrs.get(FacilityAttributeConstant.VIRTSOLUTION) != null){
						ejb.setVirtSolution((String)attrs.get(FacilityAttributeConstant.VIRTSOLUTION).get(0));
					}
				}
			} catch (javax.naming.NamingException ex) {
				if (ex instanceof CommunicationException) {
					LdapConnectionManager.getConnectionManager().setDirContext(null);
				}
					
				// NamingExceptionが発生した場合は、catchしリトライする
				m_log.debug("load(): catch NamingException" + ex.getMessage());
				
				// whileループの判定の前に、メッソッドを抜けるかのチェックをするため、
				// ここでカウントアップする。
				retryCount++;
				if(retryCount >= MAX_RETRY){
					throw new EJBException(ex);
				} else {
					//リトライ上限以下の場合には再度LDAPに問い合わせをします。
					//whlie loopに戻ります。
					continue;
				}
			} catch (NullPointerException e) {
	//				jreのバグ（？）により、NullPointerExceptionが発生することがある。
				//根本対処の方法がないため、NullPointerExcetionをcatchする。（2007/06/19　宮本）
				m_log.debug("load(): LDAP query result is null");
			
				// whileループの判定の前に、メッソッドを抜けるかのチェックをするため、
				// ここでカウントアップする。
				retryCount++;
				if(retryCount >= MAX_RETRY){
					throw new EJBException(e);
				} else {
					//リトライ上限以下の場合には再度LDAPに問い合わせをします。
					//whlie loopに戻ります。
					continue;
				}
			} finally {
				try {
					if (ctx != null) {
						ctx.close();
					}
				} catch (NamingException e) {
					m_log.error("load() context close failed : " + e.getMessage());
				}
			}
			return;
		}

	}
	/* (non-Javadoc)
	 * @see com.clustercontrol.repository.dao.FacilityDAO#store(com.clustercontrol.repository.ejb.FacilityBean)
	 */
	public void store(FacilityBean ejb) throws EJBException {
		javax.naming.directory.DirContext ctx = null;
		
		int retryCount=0;
		while(MAX_RETRY > retryCount){
				
			try {
				// LDAP用のコンテキストの作成
				ctx = LdapConnectionManager.getConnectionManager().getDirContext();
				
				//dnをキーとしてLDAPアクセスし、属性を取得
				javax.naming.directory.Attributes attrs = ctx.getAttributes(ejb.getDn());
				
				javax.naming.directory.Attributes addAttrs = 
					new javax.naming.directory.BasicAttributes();
				javax.naming.directory.Attributes replaceAttrs = 
					new javax.naming.directory.BasicAttributes();
				javax.naming.directory.Attributes removeAttrs = 
					new javax.naming.directory.BasicAttributes();
				
				//cn設定
				if(ejb.getCn() != null){
					replaceAttrs.put(FacilityAttributeConstant.CN, ejb.getCn());
				}
				else if(ejb.getCn() == null){
					removeAttrs.put(FacilityAttributeConstant.CN, null);
				}
				//description設定
				if(attrs.get(FacilityAttributeConstant.DESCRIPTION) == null && ejb.getDescription() != null){
					addAttrs.put(FacilityAttributeConstant.DESCRIPTION, ejb.getDescription());
				}
				else if(attrs.get(FacilityAttributeConstant.DESCRIPTION) != null && ejb.getDescription() == null){
					removeAttrs.put(FacilityAttributeConstant.DESCRIPTION, null);
				}
				else if(attrs.get(FacilityAttributeConstant.DESCRIPTION) != null && ejb.getDescription() != null){
					replaceAttrs.put(FacilityAttributeConstant.DESCRIPTION, ejb.getDescription());
				}
				
				if(ejb.getType() == FacilityConstant.TYPE_NODE){
					//ccAgentControl設定
					if(attrs.get(FacilityAttributeConstant.AGENTCONTROL) == null && ejb.getAgentControl() != null){
						addAttrs.put(FacilityAttributeConstant.AGENTCONTROL, booleanToString(ejb.getAgentControl()));
					}
					else if(attrs.get(FacilityAttributeConstant.AGENTCONTROL) != null && ejb.getAgentControl() == null){
						removeAttrs.put(FacilityAttributeConstant.AGENTCONTROL, null);
					}
					else if(attrs.get(FacilityAttributeConstant.AGENTCONTROL) != null && ejb.getAgentControl() != null){
						replaceAttrs.put(FacilityAttributeConstant.AGENTCONTROL, booleanToString(ejb.getAgentControl()));
					}
					//ccAgentWatch設定
					if(attrs.get(FacilityAttributeConstant.AGENTWATCH) == null && ejb.getAgentWatch() != null){
						addAttrs.put(FacilityAttributeConstant.AGENTWATCH, booleanToString(ejb.getAgentWatch()));
					}
					else if(attrs.get(FacilityAttributeConstant.AGENTWATCH) != null && ejb.getAgentWatch() == null){
						removeAttrs.put(FacilityAttributeConstant.AGENTWATCH, null);
					}
					else if(attrs.get(FacilityAttributeConstant.AGENTWATCH) != null && ejb.getAgentWatch() != null){
						replaceAttrs.put(FacilityAttributeConstant.AGENTWATCH, booleanToString(ejb.getAgentWatch()));
					}
					//ccAgentWatchStatus設定
					if(attrs.get(FacilityAttributeConstant.AGENTWATCHSTATUS) == null && ejb.getAgentWatchStatus() != null){
						addAttrs.put(FacilityAttributeConstant.AGENTWATCHSTATUS, booleanToString(ejb.getAgentWatchStatus()));
					}
					else if(attrs.get(FacilityAttributeConstant.AGENTWATCHSTATUS) != null && ejb.getAgentWatchStatus() == null){
						removeAttrs.put(FacilityAttributeConstant.AGENTWATCHSTATUS, null);
					}
					else if(attrs.get(FacilityAttributeConstant.AGENTWATCHSTATUS) != null && ejb.getAgentWatchStatus() != null){
						replaceAttrs.put(FacilityAttributeConstant.AGENTWATCHSTATUS, booleanToString(ejb.getAgentWatchStatus()));
					}
					//ccCharSet設定
					if(attrs.get(FacilityAttributeConstant.CHARSET) == null && ejb.getCharSet() != null){
						addAttrs.put(FacilityAttributeConstant.CHARSET, ejb.getCharSet());
					}
					else if(attrs.get(FacilityAttributeConstant.CHARSET) != null && ejb.getCharSet() == null){
						removeAttrs.put(FacilityAttributeConstant.CHARSET, null);
					}
					else if(attrs.get(FacilityAttributeConstant.CHARSET) != null && ejb.getCharSet() != null){
						replaceAttrs.put(FacilityAttributeConstant.CHARSET, ejb.getCharSet());
					}
					//ccCreateTimestamp設定
					if(attrs.get(FacilityAttributeConstant.CREATETIMESTAMP) == null && ejb.getCreateTimestamp() != null){
						addAttrs.put(FacilityAttributeConstant.CREATETIMESTAMP, dateToString(ejb.getCreateTimestamp()));
					}
					else if(attrs.get(FacilityAttributeConstant.CREATETIMESTAMP) != null && ejb.getCreateTimestamp() == null){
						removeAttrs.put(FacilityAttributeConstant.CREATETIMESTAMP, null);
					}
					else if(attrs.get(FacilityAttributeConstant.CREATETIMESTAMP) != null && ejb.getCreateTimestamp() != null){
						replaceAttrs.put(FacilityAttributeConstant.CREATETIMESTAMP, dateToString(ejb.getCreateTimestamp()));
					}
					//ccCreatorsName設定
					if(attrs.get(FacilityAttributeConstant.CREATORSNAME) == null && ejb.getCreatorsName() != null){
						addAttrs.put(FacilityAttributeConstant.CREATORSNAME, ejb.getCreatorsName());
					}
					else if(attrs.get(FacilityAttributeConstant.CREATORSNAME) != null && ejb.getCreatorsName() == null){
						removeAttrs.put(FacilityAttributeConstant.CREATORSNAME, null);
					}
					else if(attrs.get(FacilityAttributeConstant.CREATORSNAME) != null && ejb.getCreatorsName() != null){
						replaceAttrs.put(FacilityAttributeConstant.CREATORSNAME, ejb.getCreatorsName());
					}
					//ccValid設定
					if(attrs.get(FacilityAttributeConstant.VALID) == null && ejb.getValid() != null){
						addAttrs.put(FacilityAttributeConstant.VALID, booleanToString(ejb.getValid()));
					}
					else if(attrs.get(FacilityAttributeConstant.VALID) != null && ejb.getValid() == null){
						removeAttrs.put(FacilityAttributeConstant.VALID, null);
					}
					else if(attrs.get(FacilityAttributeConstant.VALID) != null && ejb.getValid() != null){
						replaceAttrs.put(FacilityAttributeConstant.VALID, booleanToString(ejb.getValid()));
					}
					//ccDhcpClient設定
					if(attrs.get(FacilityAttributeConstant.DHCPCLIENT) == null && ejb.getDhcpClient() != null){
						addAttrs.put(FacilityAttributeConstant.DHCPCLIENT, booleanToString(ejb.getDhcpClient()));
					}
					else if(attrs.get(FacilityAttributeConstant.DHCPCLIENT) != null && ejb.getDhcpClient() == null){
						removeAttrs.put(FacilityAttributeConstant.DHCPCLIENT, null);
					}
					else if(attrs.get(FacilityAttributeConstant.DHCPCLIENT) != null && ejb.getDhcpClient() != null){
						replaceAttrs.put(FacilityAttributeConstant.DHCPCLIENT, booleanToString(ejb.getDhcpClient()));
					}
					//host設定
					if(attrs.get(FacilityAttributeConstant.HOST) == null && ejb.getHost() != null){
						javax.naming.directory.Attribute attr =
							new javax.naming.directory.BasicAttribute(FacilityAttributeConstant.HOST);
						ArrayList list = ejb.getHost();
						for(int i = 0; i < list.size(); i++) {
							String host = (String)list.get(i);
							attr.add(i, host);
						}
						addAttrs.put(attr);
					}
					else if(attrs.get(FacilityAttributeConstant.HOST) != null && ejb.getHost() == null){
						removeAttrs.put(FacilityAttributeConstant.HOST, null);
					}
					else if(attrs.get(FacilityAttributeConstant.HOST) != null && ejb.getHost() != null){
						javax.naming.directory.Attribute attr =
							new javax.naming.directory.BasicAttribute(FacilityAttributeConstant.HOST);
						ArrayList list = ejb.getHost();
						for(int i = 0; i < list.size(); i++) {
							String host = (String)list.get(i);
							attr.add(i, host);
						}
						replaceAttrs.put(attr);
					}
					//ipNetworkNumber設定
					if(attrs.get(FacilityAttributeConstant.IPNETWORKNUMBER) == null && ejb.getIpNetworkNumber() != null){
						addAttrs.put(FacilityAttributeConstant.IPNETWORKNUMBER, ejb.getIpNetworkNumber());
					}
					else if(attrs.get(FacilityAttributeConstant.IPNETWORKNUMBER) != null && ejb.getIpNetworkNumber() == null){
						removeAttrs.put(FacilityAttributeConstant.IPNETWORKNUMBER, null);
					}
					else if(attrs.get(FacilityAttributeConstant.IPNETWORKNUMBER) != null && ejb.getIpNetworkNumber() != null){
						replaceAttrs.put(FacilityAttributeConstant.IPNETWORKNUMBER, ejb.getIpNetworkNumber());
					}
					//ccIpNetworkNumberV6設定
					if(attrs.get(FacilityAttributeConstant.IPNETWORKNUMBERV6) == null && ejb.getIpNetworkNumberV6() != null){
						addAttrs.put(FacilityAttributeConstant.IPNETWORKNUMBERV6, ejb.getIpNetworkNumberV6());
					}
					else if(attrs.get(FacilityAttributeConstant.IPNETWORKNUMBERV6) != null && ejb.getIpNetworkNumberV6() == null){
						removeAttrs.put(FacilityAttributeConstant.IPNETWORKNUMBERV6, null);
					}
					else if(attrs.get(FacilityAttributeConstant.IPNETWORKNUMBERV6) != null && ejb.getIpNetworkNumberV6() != null){
						replaceAttrs.put(FacilityAttributeConstant.IPNETWORKNUMBERV6, ejb.getIpNetworkNumberV6());
					}
					//ipProtocolNumber設定
					if(attrs.get(FacilityAttributeConstant.IPPROTOCOLNUMBER) == null && ejb.getIpProtocolNumber() != null){
						addAttrs.put(FacilityAttributeConstant.IPPROTOCOLNUMBER, ejb.getIpProtocolNumber().toString());
					}
					else if(attrs.get(FacilityAttributeConstant.IPPROTOCOLNUMBER) != null && ejb.getIpProtocolNumber() == null){
						removeAttrs.put(FacilityAttributeConstant.IPPROTOCOLNUMBER, null);
					}
					else if(attrs.get(FacilityAttributeConstant.IPPROTOCOLNUMBER) != null && ejb.getIpProtocolNumber() != null){
						replaceAttrs.put(FacilityAttributeConstant.IPPROTOCOLNUMBER, ejb.getIpProtocolNumber().toString());
					}
					//ccIpType設定
					if(attrs.get(FacilityAttributeConstant.IPTYPE) == null && ejb.getIpType() != null){
						addAttrs.put(FacilityAttributeConstant.IPTYPE, ejb.getIpType().toString());
					}
					else if(attrs.get(FacilityAttributeConstant.IPTYPE) != null && ejb.getIpType() == null){
						removeAttrs.put(FacilityAttributeConstant.IPTYPE, null);
					}
					else if(attrs.get(FacilityAttributeConstant.IPTYPE) != null && ejb.getIpType() != null){
						replaceAttrs.put(FacilityAttributeConstant.IPTYPE, ejb.getIpType().toString());
					}
					//ccLogWatch設定
					if(attrs.get(FacilityAttributeConstant.LOGWATCH) == null && ejb.getLogWatch() != null){
						addAttrs.put(FacilityAttributeConstant.LOGWATCH, booleanToString(ejb.getLogWatch()));
					}
					else if(attrs.get(FacilityAttributeConstant.LOGWATCH) != null && ejb.getLogWatch() == null){
						removeAttrs.put(FacilityAttributeConstant.LOGWATCH, null);
					}
					else if(attrs.get(FacilityAttributeConstant.LOGWATCH) != null && ejb.getLogWatch() != null){
						replaceAttrs.put(FacilityAttributeConstant.LOGWATCH, booleanToString(ejb.getLogWatch()));
					}
					//ccMachine設定
					if(attrs.get(FacilityAttributeConstant.MACHINE) == null && ejb.getMachine() != null){
						addAttrs.put(FacilityAttributeConstant.MACHINE, ejb.getMachine());
					}
					else if(attrs.get(FacilityAttributeConstant.MACHINE) != null && ejb.getMachine() == null){
						removeAttrs.put(FacilityAttributeConstant.MACHINE, null);
					}
					else if(attrs.get(FacilityAttributeConstant.MACHINE) != null && ejb.getMachine() != null){
						replaceAttrs.put(FacilityAttributeConstant.MACHINE, ejb.getMachine());
					}
					//ccManagerContact設定
					if(attrs.get(FacilityAttributeConstant.MANAGERCONTACT) == null && ejb.getManagerContact() != null){
						addAttrs.put(FacilityAttributeConstant.MANAGERCONTACT, ejb.getManagerContact());
					}
					else if(attrs.get(FacilityAttributeConstant.MANAGERCONTACT) != null && ejb.getManagerContact() == null){
						removeAttrs.put(FacilityAttributeConstant.MANAGERCONTACT, null);
					}
					else if(attrs.get(FacilityAttributeConstant.MANAGERCONTACT) != null && ejb.getManagerContact() != null){
						replaceAttrs.put(FacilityAttributeConstant.MANAGERCONTACT, ejb.getManagerContact());
					}
					//ccManagerName設定
					if(attrs.get(FacilityAttributeConstant.MANAGERNAME) == null && ejb.getManagerName() != null){
						addAttrs.put(FacilityAttributeConstant.MANAGERNAME, ejb.getManagerName());
					}
					else if(attrs.get(FacilityAttributeConstant.MANAGERNAME) != null && ejb.getManagerName() == null){
						removeAttrs.put(FacilityAttributeConstant.MANAGERNAME, null);
					}
					else if(attrs.get(FacilityAttributeConstant.MANAGERNAME) != null && ejb.getManagerName() != null){
						replaceAttrs.put(FacilityAttributeConstant.MANAGERNAME, ejb.getManagerName());
					}
					//ccModifiersName設定
					if(attrs.get(FacilityAttributeConstant.MODIFIERSNAME) == null && ejb.getModifiersName() != null){
						addAttrs.put(FacilityAttributeConstant.MODIFIERSNAME, ejb.getModifiersName());
					}
					else if(attrs.get(FacilityAttributeConstant.MODIFIERSNAME) != null && ejb.getModifiersName() == null){
						removeAttrs.put(FacilityAttributeConstant.MODIFIERSNAME, null);
					}
					else if(attrs.get(FacilityAttributeConstant.MODIFIERSNAME) != null && ejb.getModifiersName() != null){
						replaceAttrs.put(FacilityAttributeConstant.MODIFIERSNAME, ejb.getModifiersName());
					}
					//ccModifyTimestamp設定
					if(attrs.get(FacilityAttributeConstant.MODIFYTIMESTAMP) == null && ejb.getModifyTimestamp() != null){
						addAttrs.put(FacilityAttributeConstant.MODIFYTIMESTAMP, dateToString(ejb.getModifyTimestamp()));
					}
					else if(attrs.get(FacilityAttributeConstant.MODIFYTIMESTAMP) != null && ejb.getModifyTimestamp() == null){
						removeAttrs.put(FacilityAttributeConstant.MODIFYTIMESTAMP, null);
					}
					else if(attrs.get(FacilityAttributeConstant.MODIFYTIMESTAMP) != null && ejb.getModifyTimestamp() != null){
						replaceAttrs.put(FacilityAttributeConstant.MODIFYTIMESTAMP, dateToString(ejb.getModifyTimestamp()));
					}
					//ccNodeName設定
					if(attrs.get(FacilityAttributeConstant.NODENAME) == null && ejb.getNodeName() != null){
						addAttrs.put(FacilityAttributeConstant.NODENAME, ejb.getNodeName());
					}
					else if(attrs.get(FacilityAttributeConstant.NODENAME) != null && ejb.getNodeName() == null){
						removeAttrs.put(FacilityAttributeConstant.NODENAME, null);
					}
					else if(attrs.get(FacilityAttributeConstant.NODENAME) != null && ejb.getNodeName() != null){
						replaceAttrs.put(FacilityAttributeConstant.NODENAME, ejb.getNodeName());
					}
					//ccOsName設定
					if(attrs.get(FacilityAttributeConstant.OSNAME) == null && ejb.getOsName() != null){
						addAttrs.put(FacilityAttributeConstant.OSNAME, ejb.getOsName());
					}
					else if(attrs.get(FacilityAttributeConstant.OSNAME) != null && ejb.getOsName() == null){
						removeAttrs.put(FacilityAttributeConstant.OSNAME, null);
					}
					else if(attrs.get(FacilityAttributeConstant.OSNAME) != null && ejb.getOsName() != null){
						replaceAttrs.put(FacilityAttributeConstant.OSNAME, ejb.getOsName());
					}
					//ccOsRelease設定
					if(attrs.get(FacilityAttributeConstant.OSRELEASE) == null && ejb.getOsRelease() != null){
						addAttrs.put(FacilityAttributeConstant.OSRELEASE, ejb.getOsRelease());
					}
					else if(attrs.get(FacilityAttributeConstant.OSRELEASE) != null && ejb.getOsRelease() == null){
						removeAttrs.put(FacilityAttributeConstant.OSRELEASE, null);
					}
					else if(attrs.get(FacilityAttributeConstant.OSRELEASE) != null && ejb.getOsRelease() != null){
						replaceAttrs.put(FacilityAttributeConstant.OSRELEASE, ejb.getOsRelease());
					}
					//ccOsVersion設定
					if(attrs.get(FacilityAttributeConstant.OSVERSION) == null && ejb.getOsVersion() != null){
						addAttrs.put(FacilityAttributeConstant.OSVERSION, ejb.getOsVersion());
					}
					else if(attrs.get(FacilityAttributeConstant.OSVERSION) != null && ejb.getOsVersion() == null){
						removeAttrs.put(FacilityAttributeConstant.OSVERSION, null);
					}
					else if(attrs.get(FacilityAttributeConstant.OSVERSION) != null && ejb.getOsVersion() != null){
						replaceAttrs.put(FacilityAttributeConstant.OSVERSION, ejb.getOsVersion());
					}
					//ccPingWatch設定
					if(attrs.get(FacilityAttributeConstant.PINGWATCH) == null && ejb.getPingWatch() != null){
						addAttrs.put(FacilityAttributeConstant.PINGWATCH, booleanToString(ejb.getPingWatch()));
					}
					else if(attrs.get(FacilityAttributeConstant.PINGWATCH) != null && ejb.getPingWatch() == null){
						removeAttrs.put(FacilityAttributeConstant.PINGWATCH, null);
					}
					else if(attrs.get(FacilityAttributeConstant.PINGWATCH) != null && ejb.getPingWatch() != null){
						replaceAttrs.put(FacilityAttributeConstant.PINGWATCH, booleanToString(ejb.getPingWatch()));
					}
					//ccPingWatchStatus設定
					if(attrs.get(FacilityAttributeConstant.PINGWATCHSTATUS) == null && ejb.getPingWatchStatus() != null){
						addAttrs.put(FacilityAttributeConstant.PINGWATCHSTATUS, booleanToString(ejb.getPingWatchStatus()));
					}
					else if(attrs.get(FacilityAttributeConstant.PINGWATCHSTATUS) != null && ejb.getPingWatchStatus() == null){
						removeAttrs.put(FacilityAttributeConstant.PINGWATCHSTATUS, null);
					}
					else if(attrs.get(FacilityAttributeConstant.PINGWATCHSTATUS) != null && ejb.getPingWatchStatus() != null){
						replaceAttrs.put(FacilityAttributeConstant.PINGWATCHSTATUS, booleanToString(ejb.getPingWatchStatus()));
					}
					//ccPlatform設定
					if(attrs.get(FacilityAttributeConstant.PLATFORM) == null && ejb.getPlatform() != null){
						addAttrs.put(FacilityAttributeConstant.PLATFORM, ejb.getPlatform());
					}
					else if(attrs.get(FacilityAttributeConstant.PLATFORM) != null && ejb.getPlatform() == null){
						removeAttrs.put(FacilityAttributeConstant.PLATFORM, null);
					}
					else if(attrs.get(FacilityAttributeConstant.PLATFORM) != null && ejb.getPlatform() != null){
						replaceAttrs.put(FacilityAttributeConstant.PLATFORM, ejb.getPlatform());
					}
					//ccStatusWatch設定
					if(attrs.get(FacilityAttributeConstant.STATUSWATCH) == null && ejb.getStatusWatch() != null){
						addAttrs.put(FacilityAttributeConstant.STATUSWATCH, booleanToString(ejb.getStatusWatch()));
					}
					else if(attrs.get(FacilityAttributeConstant.STATUSWATCH) != null && ejb.getStatusWatch() == null){
						removeAttrs.put(FacilityAttributeConstant.STATUSWATCH, null);
					}
					else if(attrs.get(FacilityAttributeConstant.STATUSWATCH) != null && ejb.getStatusWatch() != null){
						replaceAttrs.put(FacilityAttributeConstant.STATUSWATCH, booleanToString(ejb.getStatusWatch()));
					}
					//ccSnmpPort設定
					if(attrs.get(FacilityAttributeConstant.SNMPPORT) == null && ejb.getSnmpPort() != null){
						addAttrs.put(FacilityAttributeConstant.SNMPPORT, ejb.getSnmpPort().toString());
					}
					else if(attrs.get(FacilityAttributeConstant.SNMPPORT) != null && ejb.getSnmpPort() == null){
						removeAttrs.put(FacilityAttributeConstant.SNMPPORT, null);
					}
					else if(attrs.get(FacilityAttributeConstant.SNMPPORT) != null && ejb.getSnmpPort() != null){
						replaceAttrs.put(FacilityAttributeConstant.SNMPPORT, ejb.getSnmpPort().toString());
					}
					//ccSnmpCommunity設定
					if(attrs.get(FacilityAttributeConstant.SNMPCOMMUNITY) == null && ejb.getSnmpCommunity() != null){
						addAttrs.put(FacilityAttributeConstant.SNMPCOMMUNITY, ejb.getSnmpCommunity());
					}
					else if(attrs.get(FacilityAttributeConstant.SNMPCOMMUNITY) != null && ejb.getSnmpCommunity() == null){
						removeAttrs.put(FacilityAttributeConstant.SNMPCOMMUNITY, null);
					}
					else if(attrs.get(FacilityAttributeConstant.SNMPCOMMUNITY) != null && ejb.getSnmpCommunity() != null){
						replaceAttrs.put(FacilityAttributeConstant.SNMPCOMMUNITY, ejb.getSnmpCommunity());
					}
					//ccSnmpVersion設定
					if(attrs.get(FacilityAttributeConstant.SNMPVERSION) == null && ejb.getSnmpVersion() != null){
						addAttrs.put(FacilityAttributeConstant.SNMPVERSION, ejb.getSnmpVersion());
					}
					else if(attrs.get(FacilityAttributeConstant.SNMPVERSION) != null && ejb.getSnmpVersion() == null){
						removeAttrs.put(FacilityAttributeConstant.SNMPVERSION, null);
					}
					else if(attrs.get(FacilityAttributeConstant.SNMPVERSION) != null && ejb.getSnmpVersion() != null){
						replaceAttrs.put(FacilityAttributeConstant.SNMPVERSION, ejb.getSnmpVersion());
					}
					
					//ccSortValue設定
					if(attrs.get(FacilityAttributeConstant.SORT_VALUE) == null && ejb.getSortValue() != null){
						addAttrs.put(FacilityAttributeConstant.SORT_VALUE, ejb.getSortValue().toString());
					}
					else if(attrs.get(FacilityAttributeConstant.SORT_VALUE) != null && ejb.getSortValue() == null){
						removeAttrs.put(FacilityAttributeConstant.SORT_VALUE, null);
					}
					else if(attrs.get(FacilityAttributeConstant.SORT_VALUE) != null && ejb.getSortValue() != null){
						replaceAttrs.put(FacilityAttributeConstant.SORT_VALUE, ejb.getSortValue().toString());
					}
					
					
					
					//ccSnmpTimeout設定
					if(attrs.get(FacilityAttributeConstant.SNMPTIMEOUT) == null && ejb.getSnmpTimeout() != null){
						addAttrs.put(FacilityAttributeConstant.SNMPTIMEOUT, ejb.getSnmpTimeout().toString());
					}
					else if(attrs.get(FacilityAttributeConstant.SNMPTIMEOUT) != null && ejb.getSnmpTimeout() == null){
						removeAttrs.put(FacilityAttributeConstant.SNMPTIMEOUT, null);
					}
					else if(attrs.get(FacilityAttributeConstant.SNMPTIMEOUT) != null && ejb.getSnmpTimeout() != null){
						replaceAttrs.put(FacilityAttributeConstant.SNMPTIMEOUT, ejb.getSnmpTimeout().toString());
					}
					
					//ccSnmpRetries設定
					if(attrs.get(FacilityAttributeConstant.SNMPRETRIES) == null && ejb.getSnmpRetries() != null){
						addAttrs.put(FacilityAttributeConstant.SNMPRETRIES, ejb.getSnmpRetries().toString());
					}
					else if(attrs.get(FacilityAttributeConstant.SNMPRETRIES) != null && ejb.getSnmpRetries() == null){
						removeAttrs.put(FacilityAttributeConstant.SNMPRETRIES, null);
					}
					else if(attrs.get(FacilityAttributeConstant.SNMPRETRIES) != null && ejb.getSnmpRetries() != null){
						replaceAttrs.put(FacilityAttributeConstant.SNMPRETRIES, ejb.getSnmpRetries().toString());
					}
					
					//ccSnmpProxy設定
					if(attrs.get(FacilityAttributeConstant.SNMPPROXY) == null && ejb.getSnmpProxy() != null){
						addAttrs.put(FacilityAttributeConstant.SNMPPROXY, ejb.getSnmpProxy().toString());
					}
					else if(attrs.get(FacilityAttributeConstant.SNMPPROXY) != null && ejb.getSnmpProxy() == null){
						removeAttrs.put(FacilityAttributeConstant.SNMPPROXY, null);
					}
					else if(attrs.get(FacilityAttributeConstant.SNMPPROXY) != null && ejb.getSnmpProxy() != null){
						replaceAttrs.put(FacilityAttributeConstant.SNMPPROXY, ejb.getSnmpProxy().toString());
					}
					
					//ccNote設定
					if(attrs.get(FacilityAttributeConstant.NOTE) == null && ejb.getNote() != null){
						javax.naming.directory.Attribute attr =
							new javax.naming.directory.BasicAttribute(FacilityAttributeConstant.NOTE);
						ArrayList list = ejb.getNote();
						for(int i = 0; i < list.size(); i++) {
							String note = (String)list.get(i);
							attr.add(i, note);
						}
						addAttrs.put(attr);
					}
					else if(attrs.get(FacilityAttributeConstant.NOTE) != null && ejb.getNote() == null){
						removeAttrs.put(FacilityAttributeConstant.NOTE, null);
					}
					else if(attrs.get(FacilityAttributeConstant.NOTE) != null && ejb.getNote() != null){
						javax.naming.directory.Attribute attr =
							new javax.naming.directory.BasicAttribute(FacilityAttributeConstant.NOTE);
						ArrayList list = ejb.getNote();
						for(int i = 0; i < list.size(); i++) {
							String note = (String)list.get(i);
							attr.add(i, note);
						}
						replaceAttrs.put(attr);
					}
					
					//ccIconImage設定
					if(attrs.get(FacilityAttributeConstant.ICONIMAGE) == null && ejb.getIconImage() != null){
						addAttrs.put(FacilityAttributeConstant.ICONIMAGE, ejb.getIconImage());
					}
					else if(attrs.get(FacilityAttributeConstant.ICONIMAGE) != null && ejb.getIconImage() == null){
						removeAttrs.put(FacilityAttributeConstant.ICONIMAGE, null);
					}
					else if(attrs.get(FacilityAttributeConstant.ICONIMAGE) != null && ejb.getIconImage() != null){
						replaceAttrs.put(FacilityAttributeConstant.ICONIMAGE, ejb.getIconImage());
					}
					
					//ccVirtualizationNoteType設定
					if(attrs.get(FacilityAttributeConstant.VIRTNODETYPE) == null && ejb.getVirtNodeType() != null){
						addAttrs.put(FacilityAttributeConstant.VIRTNODETYPE, ejb.getVirtNodeType().toString());
					}
					else if(attrs.get(FacilityAttributeConstant.VIRTNODETYPE) != null && ejb.getVirtNodeType() == null){
						removeAttrs.put(FacilityAttributeConstant.VIRTNODETYPE, null);
					}
					else if(attrs.get(FacilityAttributeConstant.VIRTNODETYPE) != null && ejb.getVirtNodeType() != null){
						replaceAttrs.put(FacilityAttributeConstant.VIRTNODETYPE, ejb.getVirtNodeType().toString());
					}
					
					//ccVMManagementNode設定
					if(attrs.get(FacilityAttributeConstant.VMMANAGEMENTNODE) == null && ejb.getVMManagementNode() != null){
						addAttrs.put(FacilityAttributeConstant.VMMANAGEMENTNODE, ejb.getVMManagementNode().toString());
					}
					else if(attrs.get(FacilityAttributeConstant.VMMANAGEMENTNODE) != null && ejb.getVMManagementNode() == null){
						removeAttrs.put(FacilityAttributeConstant.VMMANAGEMENTNODE, null);
					}
					else if(attrs.get(FacilityAttributeConstant.VMMANAGEMENTNODE) != null && ejb.getVMManagementNode() != null){
						replaceAttrs.put(FacilityAttributeConstant.VMMANAGEMENTNODE, ejb.getVMManagementNode().toString());
					}
					
					//ccVMID設定
					if(attrs.get(FacilityAttributeConstant.VMINDEX) == null && ejb.getVmIndex() != null){
						addAttrs.put(FacilityAttributeConstant.VMINDEX, ejb.getVmIndex().toString());
					}
					else if(attrs.get(FacilityAttributeConstant.VMINDEX) != null && ejb.getVmIndex() == null){
						removeAttrs.put(FacilityAttributeConstant.VMINDEX, null);
					}
					else if(attrs.get(FacilityAttributeConstant.VMINDEX) != null && ejb.getVmIndex() != null){
						replaceAttrs.put(FacilityAttributeConstant.VMINDEX, ejb.getVmIndex().toString());
					}
					
					//ccVMName設定
					if(attrs.get(FacilityAttributeConstant.VMNAME) == null && ejb.getVmName() != null){
						addAttrs.put(FacilityAttributeConstant.VMNAME, ejb.getVmName().toString());
					}
					else if(attrs.get(FacilityAttributeConstant.VMNAME) != null && ejb.getVmName() == null){
						removeAttrs.put(FacilityAttributeConstant.VMNAME, null);
					}
					else if(attrs.get(FacilityAttributeConstant.VMNAME) != null && ejb.getVmName() != null){
						replaceAttrs.put(FacilityAttributeConstant.VMNAME, ejb.getVmName().toString());
					}
					
					//ccVirtualizationSolution設定
					if(attrs.get(FacilityAttributeConstant.VIRTSOLUTION) == null && ejb.getVirtSolution() != null){
						addAttrs.put(FacilityAttributeConstant.VIRTSOLUTION, ejb.getVirtSolution().toString());
					}
					else if(attrs.get(FacilityAttributeConstant.VIRTSOLUTION) != null && ejb.getVirtSolution() == null){
						removeAttrs.put(FacilityAttributeConstant.VIRTSOLUTION, null);
					}
					else if(attrs.get(FacilityAttributeConstant.VIRTSOLUTION) != null && ejb.getVirtSolution() != null){
						replaceAttrs.put(FacilityAttributeConstant.VIRTSOLUTION, ejb.getVirtSolution().toString());
					}

				}
				//属性を変更する（追加）
				ctx.modifyAttributes(
						ejb.getDn(),
						javax.naming.directory.DirContext.ADD_ATTRIBUTE, 
						addAttrs);
				//属性を変更する（置換）
				ctx.modifyAttributes(
						ejb.getDn(), 
						javax.naming.directory.DirContext.REPLACE_ATTRIBUTE, 
						replaceAttrs);
				//属性を変更する（削除）
				ctx.modifyAttributes(
						ejb.getDn(), 
						javax.naming.directory.DirContext.REMOVE_ATTRIBUTE, 
						removeAttrs);
				
			} catch (javax.naming.NamingException ex) {
				if (ex instanceof CommunicationException) {
						LdapConnectionManager.getConnectionManager().setDirContext(null);
				}
					
				// NamingExceptionが発生した場合は、catchしリトライする
				m_log.debug("store(): catch NamingException" + ex.getMessage());
				
				// whileループの判定の前に、メッソッドを抜けるかのチェックをするため、
				// ここでカウントアップする。
				retryCount++;
				if(retryCount >= MAX_RETRY){
					throw new EJBException(ex);
				} else {
					//リトライ上限以下の場合には再度LDAPに問い合わせをします。
					//whlie loopに戻ります。
					continue;
				}
				
			} catch (NullPointerException e) {
				//jreのバグ（？）により、NullPointerExceptionが発生することがある。
				//根本対処の方法がないため、NullPointerExcetionをcatchする。（2007/06/19　宮本）
				m_log.debug("store(): LDAP query result is null");
			
				// whileループの判定の前に、メッソッドを抜けるかのチェックをするため、
				// ここでカウントアップする。
				retryCount++;
				if(retryCount >= MAX_RETRY){
					throw new EJBException(e);
				} else {
					//リトライ上限以下の場合には再度LDAPに問い合わせをします。
					//whlie loopに戻ります。
					continue;
				}
				
			} finally {
				try {
					if (ctx != null) {
						ctx.close();
					}
				} catch (NamingException e) {
					m_log.error("store() context close failed : " + e.getMessage());
				}
			}
			return ;
		}
	}
	
	/* (non-Javadoc)
	 * @see com.clustercontrol.repository.dao.FacilityDAO#remove(com.clustercontrol.repository.ejb.FacilityPK)
	 */
	public void remove(FacilityPK pk) throws RemoveException, EJBException {
		javax.naming.directory.DirContext ctx = null;
		
		int retryCount=0;
		while(MAX_RETRY > retryCount){
			
			try {
				// LDAP用のコンテキストの作成
				ctx = LdapConnectionManager.getConnectionManager().getDirContext();
	
				//dnをキーとしてLDAPアクセスし、エントリを削除
//				ctx.destroySubcontext(pk.getDn());
				ctx.destroySubcontext(createDnString(pk.getFacilityId()));
				
			} catch (javax.naming.NamingException ex) {
				if (ex instanceof CommunicationException) {
						LdapConnectionManager.getConnectionManager().setDirContext(null);
				}
				
				// NamingExceptionが発生した場合は、catchしリトライする
				m_log.debug("remove(): catch NamingException" + ex.getMessage());
				
				// whileループの判定の前に、メッソッドを抜けるかのチェックをするため、
				// ここでカウントアップする。
				retryCount++;
				if(retryCount >= MAX_RETRY){
					AplLogger apllog = new AplLogger("REP", "rep");
//					String[] args = { pk.getDn() };
					String[] args = { createDnString(pk.getFacilityId()) };
					apllog.put("SYS", "006", args);
					throw new EJBException(ex);
				} else {
					//リトライ上限以下の場合には再度LDAPに問い合わせをします。
					//whlie loopに戻ります。
					continue;
				}	
			}catch (NullPointerException e){
				//jreのバグ（？）により、NullPointerExceptionが発生することがある。
				//根本対処の方法がないため、NullPointerExcetionをcatchする。（2007/06/19　宮本）
				m_log.debug("remove(): LDAP query result is null");
			
				// whileループの判定の前に、メッソッドを抜けるかのチェックをするため、
				// ここでカウントアップする。
				retryCount++;
				if(retryCount >= MAX_RETRY){
					throw new EJBException(e);
				} else {
					//リトライ上限以下の場合には再度LDAPに問い合わせをします。
					//whlie loopに戻ります。
					continue;
				}
				
			} finally {
				try {
					if (ctx != null) {
						ctx.close();
					}
				} catch (NamingException e) {
					m_log.error("remove() context close failed : " + e.getMessage());
				}
			}
			return;
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.clustercontrol.repository.dao.FacilityDAO#create(com.clustercontrol.repository.ejb.FacilityBean)
	 */
	public FacilityPK create(FacilityBean ejb) throws CreateException, EJBException {
		FacilityPK pk = null;
		javax.naming.directory.DirContext ctx = null;
		
		int retryCount=0;
		while(MAX_RETRY > retryCount){
				
			try {
				// LDAP用のコンテキストの作成
				ctx = LdapConnectionManager.getConnectionManager().getDirContext();
				
				// Attributes for new entry
				javax.naming.directory.Attributes attrs =
					new javax.naming.directory.BasicAttributes();
				
				//objectClassを設定
				javax.naming.directory.Attribute attr =
					new javax.naming.directory.BasicAttribute(OBJECTCLASS);
				if(ejb.getType() == FacilityConstant.TYPE_NODE){
					attr.add(0, OBJECT_CLASS_FACILITY);
					attr.add(1, OBJECT_CLASS_NODE);
				}
				else{
					attr.add(0, OBJECT_CLASS_FACILITY);
					attr.add(1, OBJECT_CLASS_SCOPE);
				}
				attrs.put(attr);
				
				//ファシリティIDを設定
				attrs.put(FacilityAttributeConstant.FACILITYID, ejb.getFacilityId());
				//ファシリティ名を設定
				attrs.put(FacilityAttributeConstant.CN, ejb.getCn());
				
				// dn文字列作成
//				StringBuffer dn = new StringBuffer();
//				dn.append(FacilityAttributeConstant.FACILITYID);
//				dn.append("=");
//				dn.append(ejb.getFacilityId());
//				dn.append(",");
//				dn.append(BASE_DN);
				
				//dnを設定
//				ejb.setDn(dn.toString());
				ejb.setDn(createDnString(ejb.getFacilityId()));
	
				//エントリを作成
				ctx.createSubcontext(ejb.getDn(), attrs);
	
//				pk = new FacilityPK(ejb.getDn());
				pk = new FacilityPK(ejb.getFacilityId());
				
			} catch (NameAlreadyBoundException ex) {
				// ファシリティIDが重複している場合、NameAlreadyBoundExceptionが発生する
				// NameingExceptionはthrow出来ないため、DuplicateKeyExceptionとしてthrowする
				throw new DuplicateKeyException();
			} catch ( javax.naming.NamingException ex ) {
				if(ex instanceof CommunicationException){
					LdapConnectionManager.getConnectionManager().setDirContext(null);
				}			
			
				// NamingExceptionが発生した場合は、catchしリトライする
				m_log.debug("create(): catch NamingException" + ex.getMessage());
				
				// whileループの判定の前に、メッソッドを抜けるかのチェックをするため、
				// ここでカウントアップする。
				retryCount++;
				if(retryCount >= MAX_RETRY){
					AplLogger apllog = new AplLogger("REP", "rep");
					String[] args = { ejb.getDn() };
					apllog.put("SYS", "005", args);
					throw new EJBException(ex);
				} else {
					//リトライ上限以下の場合には再度LDAPに問い合わせをします。
					//whlie loopに戻ります。
					continue;
				}
				
			} catch (NullPointerException e) {
				//jreのバグ（？）により、NullPointerExceptionが発生することがある。
				//根本対処の方法がないため、NullPointerExcetionをcatchする。（2007/06/19　宮本）
				m_log.debug("create(): LDAP query result is null");
			
			
							// whileループの判定の前に、メッソッドを抜けるかのチェックをするため、
				// ここでカウントアップする。
				retryCount++;
				if(retryCount >= MAX_RETRY){
					throw new EJBException(e);
				} else {
					//リトライ上限以下の場合には再度LDAPに問い合わせをします。
					//whlie loopに戻ります。
					continue;
				}
				
	//			2.1.1改修分マージ時にコメントアウト 7/4 kozakai			
	//			if (ex instanceof CommunicationException) {
	//				LdapConnectionManager.getConnectionManager().setDirContext(null);
	//			} else if (ex instanceof NameAlreadyBoundException) {
	//				throw new DuplicateKeyException(ex.getMessage());
	//			}
	//			throw new CreateException(ex.getMessage());
				
			} finally {
				try {
					if (ctx != null) {
						ctx.close();
					}
				} catch (NamingException e) {
					m_log.error("create() context close failed : " + e.getMessage());
				}
			}
	
			return pk;
		}
		return pk;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.clustercontrol.repository.dao.FacilityDAO#findAll()
	 */
	public Collection findAll() throws FinderException {
		ArrayList ret = new ArrayList();

		m_log.debug("findAll()");
		
		// エントリの検索
		javax.naming.directory.SearchControls constraints = 
			new javax.naming.directory.SearchControls();

		// BaseDN以下にぶら下がるすべてのレベルを検索の対象
		constraints.setSearchScope(javax.naming.directory.SearchControls.SUBTREE_SCOPE);

		// 検索文字列作成
		StringBuffer search = new StringBuffer();
		search.append("(&(");
		search.append(OBJECTCLASS);
		search.append("=");
		search.append(OBJECT_CLASS_FACILITY);
		search.append("))");
		
		//検索実行
		javax.naming.NamingEnumeration results = null;
		javax.naming.directory.DirContext ctx = null;
		int retryCount=0;
		
		while(MAX_RETRY > retryCount){
			try {
				// LDAP用のコンテキストの作成
				ctx = LdapConnectionManager.getConnectionManager().getDirContext();
				
				results = ctx.search(BASE_DN, search.toString(), constraints);
				
				if (results == null) {
					m_log.debug("findAll() : No Search Result");
					return ret;
				}
	
				//検索結果を取得
				while ( results.hasMoreElements() ) {
					javax.naming.directory.SearchResult aResult = 
						(javax.naming.directory.SearchResult)results.nextElement();
//					FacilityPK pk = new FacilityPK(aResult.getName() + "," + BASE_DN);
					Attributes attrs = aResult.getAttributes();
					FacilityPK pk = new FacilityPK((String)attrs.get(FacilityAttributeConstant.FACILITYID).get(0));
					
					ret.add(pk);
				}
				
			} catch (NamingException ex) {
				if(ex instanceof CommunicationException){
		        LdapConnectionManager.getConnectionManager().setDirContext(null);
		    }
				m_log.debug("findAll() : " + ex.getMessage());
				
				//中身を空にします。
				ret.clear();
				
				// whileループの判定の前に、メッソッドを抜けるかのチェックをするため、
				// ここでカウントアップする。
				retryCount++;
				if(retryCount >= MAX_RETRY){
					throw new EJBException(ex);
				} else {
					//リトライ上限以下の場合には再度LDAPに問い合わせをします。
					//whlie loopに戻ります。
					continue;
				}
			} catch (NullPointerException e) {
				//jreのバグ（？）により、hasMoreElements()でNullPointerExceptionが発生することがある。
				//根本対処の方法がないため、NullPointerExcetionをcatchする。（2007/06/19　宮本）
				m_log.debug("findAll() : LDAP query result is null");
				
				//中身を空にします。
				ret.clear();
				
				// whileループの判定の前に、メッソッドを抜けるかのチェックをするため、
				// ここでカウントアップする。
				retryCount++;
				if(retryCount >= MAX_RETRY){
					throw new EJBException(e);
				} else {
					//リトライ上限以下の場合には再度LDAPに問い合わせをします。
					//whlie loopに戻ります。
					continue;
				}
				
			} finally {
				try {
					if(results != null)
						results.close();
					if(ctx != null)
						ctx.close();
				} catch (NamingException e) {
					m_log.error("findAll() context close failed : " + e.getMessage());
				}
			}
			
			return ret;
		}
		return ret;
	}
	
	/* (non-Javadoc)
	 * @see com.clustercontrol.repository.dao.FacilityDAO#findAllNode()
	 */
	public Collection findAllNode() throws FinderException {
		ArrayList ret = new ArrayList();

		m_log.debug("findAllNode()");
		
		// エントリの検索
		javax.naming.directory.SearchControls constraints = 
			new javax.naming.directory.SearchControls();

		// BaseDN以下にぶら下がるすべてのレベルを検索の対象
		constraints.setSearchScope(javax.naming.directory.SearchControls.SUBTREE_SCOPE);

		// 検索文字列作成
		StringBuffer search = new StringBuffer();
		search.append("(&(");
		search.append(OBJECTCLASS);
		search.append("=");
		search.append(OBJECT_CLASS_FACILITY);
		search.append(")(");
		search.append(OBJECTCLASS);
		search.append("=");
		search.append(OBJECT_CLASS_NODE);
		search.append("))");
		
		//検索実行
		javax.naming.NamingEnumeration results = null;
		javax.naming.directory.DirContext ctx = null;
		int retryCount=0;
		
		while(MAX_RETRY > retryCount){
			
			try {
				// LDAP用のコンテキストの作成
				ctx = LdapConnectionManager.getConnectionManager().getDirContext();
				
				results = ctx.search(BASE_DN, search.toString(), constraints);
				
				if (results == null ) {
						m_log.debug("findAllNode() : No Search Result");
					return null;
				}
	
				//検索結果を取得
				while ( results.hasMoreElements() ) {
					javax.naming.directory.SearchResult aResult = 
						(javax.naming.directory.SearchResult)results.nextElement();
//					FacilityPK pk = new FacilityPK(aResult.getName() + "," + BASE_DN);
					Attributes attrs = aResult.getAttributes();
					FacilityPK pk = new FacilityPK((String)attrs.get(FacilityAttributeConstant.FACILITYID).get(0));
					
					ret.add(pk);
				}
				
			} catch (NamingException ex) {
				if(ex instanceof CommunicationException){
					LdapConnectionManager.getConnectionManager().setDirContext(null);
				}
				m_log.debug("findAllNode() : " + ex.getMessage());
				
				//中身を空にします。
				ret.clear();
				
				// whileループの判定の前に、メッソッドを抜けるかのチェックをするため、
				// ここでカウントアップする。
				retryCount++;
				if(retryCount >= MAX_RETRY){
					throw new EJBException(ex);
				} else {
					//リトライ上限以下の場合には再度LDAPに問い合わせをします。
					//whlie loopに戻ります。
					continue;
				}
					
			} catch (NullPointerException e){
				//jreのバグ（？）により、hasMoreElements()でNullPointerExceptionが発生することがある。
				//根本対処の方法がないため、NullPointerExcetionをcatchする。（2007/06/19　宮本）
				m_log.debug("findAllNode() : LDAP query result is null");
				
				//中身を空にします。
				ret.clear();
				
	//				whileループの判定の前に、メッソッドを抜けるかのチェックをするため、
				// ここでカウントアップする。
				retryCount++;
				if(retryCount >= MAX_RETRY){
					throw new EJBException(e);
				} else {
					//リトライ上限以下の場合には再度LDAPに問い合わせをします。
					//whlie loopに戻ります。
					continue;
				}
			} finally {
				try {
					if(results != null)
						results.close();
					if(ctx != null)
						ctx.close();
				} catch (NamingException e) {
					m_log.error("findAllNode() context close failed : " + e.getMessage());
				}
			}
			
			return ret;
		}
		return ret;
	}
	
	/* (non-Javadoc)
	 * @see com.clustercontrol.repository.dao.FacilityDAO#findByPrimaryKey(com.clustercontrol.repository.ejb.FacilityPK)
	 */
	public FacilityPK findByPrimaryKey(FacilityPK pk) throws FinderException {
		javax.naming.directory.DirContext ctx = null;
		int retryCount=0;
		while(MAX_RETRY > retryCount){
			try {
//				m_log.debug("findByPrimaryKey() : dn = " + pk.getDn());
				m_log.debug("findByPrimaryKey() : facilityId = " + pk.getFacilityId());
				
				// LDAP用のコンテキストの作成
				ctx = LdapConnectionManager.getConnectionManager().getDirContext();
				
				//dnをキーとしてLDAPアクセスし、属性を取得
//				javax.naming.directory.Attributes attrs = ctx.getAttributes(pk.getDn());
				javax.naming.directory.Attributes attrs = ctx.getAttributes(createDnString(pk.getFacilityId()));
	//			2.1.1改修分マージ時にコメントアウト 7/5 kozakai					
	//			return pk;
				
			} catch ( javax.naming.NamingException ex ) {
			    if(ex instanceof CommunicationException){
			        LdapConnectionManager.getConnectionManager().setDirContext(null);
			    }
				m_log.debug("findByPrimaryKey() : " + ex.getMessage());
				
				// whileループの判定の前に、メッソッドを抜けるかのチェックをするため、
				// ここでカウントアップする。
				retryCount++;
				if(retryCount >= MAX_RETRY){
					throw new EJBException(ex);
				} else {
					//リトライ上限以下の場合には再度LDAPに問い合わせをします。
					//whlie loopに戻ります。
					continue;
				}
				
//				2.1.1改修分マージ時にコメントアウト 7/5 kozakai					
//				m_log.debug("findByPrimaryKey() : " + ex.getMessage());
//				throw new FinderException("FacilityDAOImpl.findByPrimariKey() : dn=" + pk.getDn() + " not found.");

				
			} catch (NullPointerException e){
				//jreのバグ（？）により、NullPointerExceptionが発生することがある。
				//根本対処の方法がないため、NullPointerExcetionをcatchする。（2007/06/19　宮本）
				m_log.debug("findAllNode() : LDAP query result is null");
	//				whileループの判定の前に、メッソッドを抜けるかのチェックをするため、
				// ここでカウントアップする。
				retryCount++;
				if(retryCount >= MAX_RETRY){
					throw new EJBException(e);
				} else {
					//リトライ上限以下の場合には再度LDAPに問い合わせをします。
					//whlie loopに戻ります。
					continue;
				}
				
			} finally {
				try {
					if(ctx != null){
						ctx.close();
					}
				} catch (NamingException e) {
					m_log.error("findByPrimaryKey() context close failed : " + e.getMessage());
				}
			}
			return pk;
		}
		return pk;
	}
	
	/* (non-Javadoc)
	 * @see com.clustercontrol.repository.dao.FacilityDAO#findByFacilityId(java.lang.String)
	 */
	public FacilityPK findByFacilityId(String facilityId) throws FinderException {
		
		m_log.debug("start");
		
		FacilityPK pk = new FacilityPK(facilityId);
		m_log.debug("end");
		return pk;
		
		
		
//		m_log.debug("findByFacilityId() : FacilityId = " + facilityId);
//		
//		// エントリの検索
//		javax.naming.directory.SearchControls constraints = 
//			new javax.naming.directory.SearchControls();
//
//		// 名前付きオブジェクトを検索
//		constraints.setSearchScope(javax.naming.directory.SearchControls.SUBTREE_SCOPE);
//
//		// 検索文字列作成
//		StringBuffer search = new StringBuffer();
//		search.append("(&(");
//		search.append(OBJECTCLASS);
//		search.append("=");
//		search.append(OBJECT_CLASS_FACILITY);
//		search.append(")(");
//		search.append(FacilityAttributeConstant.FACILITYID);
//		search.append("=");
//		search.append(facilityId);
//		search.append("))");
//		
//		//検索実行
//		javax.naming.NamingEnumeration results = null;
//		javax.naming.directory.DirContext ctx = null;
//		FacilityPK pk = null;
//		
//		int retryCount=0;
//		
//		while(MAX_RETRY > retryCount){
//			
//			try {
//				// LDAP用のコンテキストの作成
//				ctx = LdapConnectionManager.getConnectionManager().getDirContext();
//				
//				results = ctx.search(BASE_DN, search.toString(), constraints);
//				
//				if (results == null) {
//					throw new FinderException("FacilityDAOImpl.findByFacilityId() : FacilityId=" + facilityId + " not found.");
//				}
//				
//				javax.naming.directory.SearchResult aResult = null;
//				javax.naming.directory.Attributes attrs = null;
//				while ( results.hasMoreElements() ) {
//					aResult = (javax.naming.directory.SearchResult)results.nextElement();
//					attrs = aResult.getAttributes();
//					break;
//				}
//				
//				if (attrs == null) {
//					throw new FinderException("FacilityDAOImpl.findByFacilityId() : FacilityId=" + facilityId + " not found.");
//				}
//				
//				pk = new FacilityPK(aResult.getName() + "," + BASE_DN);
//	
//			} catch (NamingException ex) {
//				if(ex instanceof CommunicationException){
//					LdapConnectionManager.getConnectionManager().setDirContext(null);
//				}
//		    
//				m_log.debug("findByFacilityId() : " + ex.getMessage());
//				// whileループの判定の前に、メッソッドを抜けるかのチェックをするため、
//				// ここでカウントアップする。
//				retryCount++;
//				if(retryCount >= MAX_RETRY){
//					throw new EJBException(ex);
//				} else {
//					//リトライ上限以下の場合には再度LDAPに問い合わせをします。
//					//whlie loopに戻ります。
//					continue;
//				}
//				
////				2.1.1改修分マージ時にコメントアウト 7/5 kozakai					
////				m_log.debug("findByFacilityId() : " + e.getMessage());
////				throw new FinderException("FacilityDAOImpl.findByFacilityId() : FacilityId=" + facilityId + " not found.");
//
//				
//			} catch (NullPointerException e) {
//				//jreのバグ（？）により、hasMoreElements()でNullPointerExceptionが発生することがある。
//				//根本対処の方法がないため、NullPointerExcetionをcatchする。（2007/06/19　宮本）
//				m_log.debug("findByFacilityId() : LDAP query result is null");
//				// whileループの判定の前に、メッソッドを抜けるかのチェックをするため、
//				// ここでカウントアップする。
//				retryCount++;
//				if(retryCount >= MAX_RETRY){
//					throw new EJBException(e);
//				} else {
//					//リトライ上限以下の場合には再度LDAPに問い合わせをします。
//					//whlie loopに戻ります。
//					continue;
//				}
//			} finally {
//				try {
//					if(results != null)
//						results.close();
//					if(ctx != null)
//						ctx.close();
//				} catch (NamingException e) {
//					m_log.error("findByFacilityId() context close failed : " + e.getMessage());
//				}
//			}
//	
//			return pk;
//		}
//		return pk;
	}
	
	/* (non-Javadoc)
	 * @see com.clustercontrol.repository.dao.FacilityDAO#findNode(java.lang.String, java.lang.String, java.lang.String, java.lang.String, java.lang.String, java.lang.String, java.lang.String, java.lang.String)
	 */
	public Collection findNode(
			String facilityId, 
			String facilityName, 
			String description, 
			String ipAddressV4, 
			String ipAddressV6, 
			String osName, 
			String osRelease, 
			String managerName, 
			String managerContact) throws FinderException {
	    
		ArrayList ret = new ArrayList();
		
		m_log.debug("findNode() : FacilityId = " + facilityId + 
				", FacilityId = " + facilityId + 
				", FacilityName = " + facilityName + 
				", Description = " + description + 
				", ipAddressV4 = " + ipAddressV4 + 
				", ipAddressV6 = " + ipAddressV6 + 
				", osName = " + osName + 
				", osRelease = " + osRelease + 
				", managerName = " + managerName + 
				", managerContact = " + managerContact);
		
		// エントリの検索
		javax.naming.directory.SearchControls constraints = 
			new javax.naming.directory.SearchControls();

		// BaseDN以下にぶら下がるすべてのレベルを検索の対象
		constraints.setSearchScope(javax.naming.directory.SearchControls.SUBTREE_SCOPE);

		// 検索文字列作成
		StringBuffer search = new StringBuffer();
		search.append("(&(");
		search.append(OBJECTCLASS);
		search.append("=");
		search.append(OBJECT_CLASS_FACILITY);
		search.append(")(");
		search.append(OBJECTCLASS);
		search.append("=");
		search.append(OBJECT_CLASS_NODE);
		// ファシリティIDを検索文字列に設定
		if(facilityId != null && facilityId.compareTo("") != 0){
			search.append(")(");
			search.append(FacilityAttributeConstant.FACILITYID);
			search.append("=");
			search.append(facilityId);
		}
		// ファシリティ名を検索文字列に設定
		if(facilityName != null && facilityName.compareTo("") != 0){
			search.append(")(");
			search.append(FacilityAttributeConstant.CN);
			search.append("=");
			search.append(facilityName);
		}
		// 説明を検索文字列に設定
		if(description != null && description.compareTo("") != 0){
			search.append(")(");
			search.append(FacilityAttributeConstant.DESCRIPTION);
			search.append("=");
			search.append(description);
		}
		// IPアドレスv4を検索文字列に設定
		if(ipAddressV4 != null && ipAddressV4.compareTo("") != 0){
			search.append(")(");
			search.append(FacilityAttributeConstant.IPNETWORKNUMBER);
			search.append("=");
			search.append(ipAddressV4);
		}
		// IPアドレスv6を検索文字列に設定
		if(ipAddressV6 != null && ipAddressV6.compareTo("") != 0){
			search.append(")(");
			search.append(FacilityAttributeConstant.IPNETWORKNUMBER);
			search.append("=");
			search.append(ipAddressV6);
		}
		// OS名を検索文字列に設定
		if(osName != null && osName.compareTo("") != 0){
			search.append(")(");
			search.append(FacilityAttributeConstant.OSNAME);
			search.append("=");
			search.append(osName);
		}
		// OSリリースを検索文字列に設定
		if(osRelease != null && osRelease.compareTo("") != 0){
			search.append(")(");
			search.append(FacilityAttributeConstant.OSRELEASE);
			search.append("=");
			search.append(osRelease);
		}
		// 管理者を検索文字列に設定
		if(managerName != null && managerName.compareTo("") != 0){
			search.append(")(");
			search.append(FacilityAttributeConstant.MANAGERNAME);
			search.append("=");
			search.append(managerName);
		}
		// 連絡先を検索文字列に設定
		if(managerContact != null && managerContact.compareTo("") != 0){
			search.append(")(");
			search.append(FacilityAttributeConstant.MANAGERCONTACT);
			search.append("=");
			search.append(managerContact);
		}
		search.append("))");
		
		//検索実行
		javax.naming.NamingEnumeration results = null;
		javax.naming.directory.DirContext ctx = null;
		
		int retryCount=0;
		
		while(MAX_RETRY > retryCount){
			try {
				// LDAP用のコンテキストの作成
				ctx = LdapConnectionManager.getConnectionManager().getDirContext();
				
				results = ctx.search(BASE_DN, search.toString(), constraints);
				
				if (results == null ) {
					m_log.debug("findNode() : No Search Result : " + 
					        "dn = " + createDnString(facilityId) + 
					        ", FacilityId = " + facilityId + 
					        ", FacilityName = " + facilityName + 
					        ", Description = " + description + 
					        ", ipAddressV4 = " + ipAddressV4 + 
					        ", ipAddressV6 = " + ipAddressV6 + 
					        ", osName = " + osName + 
					        ", osRelease = " + osRelease + 
					        ", managerName = " + managerName + 
					        ", managerContact = " + managerContact);
					return ret;
				}
	
				//検索結果を取得
				while ( results.hasMoreElements() ) {
					javax.naming.directory.SearchResult aResult = 
						(javax.naming.directory.SearchResult)results.nextElement();
//					FacilityPK pk = new FacilityPK(aResult.getName() + "," + BASE_DN);
					Attributes attrs = aResult.getAttributes();
					FacilityPK pk = new FacilityPK((String)attrs.get(FacilityAttributeConstant.FACILITYID).get(0));

					ret.add(pk);
				}
				
			} catch (NamingException ex) {
				if(ex instanceof CommunicationException){
					LdapConnectionManager.getConnectionManager().setDirContext(null);
				}
				m_log.debug("findNode() : " + ex.getMessage());
				
				//中身を空にします。
				ret.clear();
				
				// whileループの判定の前に、メッソッドを抜けるかのチェックをするため、
				// ここでカウントアップする。
				retryCount++;
				if(retryCount >= MAX_RETRY){
					throw new EJBException(ex);
				} else {
					//リトライ上限以下の場合には再度LDAPに問い合わせをします。
					//whlie loopに戻ります。
					continue;
				}
					
			} catch (NullPointerException e) {
				//jreのバグ（？）により、hasMoreElements()でNullPointerExceptionが発生することがある。
				//根本対処の方法がないため、NullPointerExcetionをcatchする。（2007/06/19　宮本）
				
				//中身を空にします。
				ret.clear();
				
				m_log.debug("findNode() : LDAP query result is null");
				// whileループの判定の前に、メッソッドを抜けるかのチェックをするため、
				// ここでカウントアップする。
				retryCount++;
				if(retryCount >= MAX_RETRY){
					throw new EJBException(e);
				} else {
					//リトライ上限以下の場合には再度LDAPに問い合わせをします。
					//whlie loopに戻ります。
					continue;
				}
			} finally {
				try {
					if(results != null)
						results.close();
					if(ctx != null)
						ctx.close();
				} catch (NamingException e) {
					m_log.error("findNode() context close failed : " + e.getMessage());
				}
			}
			
			return ret;
		}
		LdapConnectionManager.getConnectionManager().setDirContext(null);
		throw new FinderException();
	}
	
	/* (non-Javadoc)
	 * @see com.clustercontrol.repository.dao.FacilityDAO#findByIpAddress(java.lang.String)
	 */
	public Collection findByIpAddress(String ipAddress) throws FinderException {
		ArrayList ret = new ArrayList();
		
		m_log.debug("findByIpAddress() : ipAddress = " + ipAddress);
		
		// エントリの検索
		javax.naming.directory.SearchControls constraints = 
			new javax.naming.directory.SearchControls();

		// BaseDN以下にぶら下がるすべてのレベルを検索の対象
		constraints.setSearchScope(javax.naming.directory.SearchControls.SUBTREE_SCOPE);

		// 検索文字列作成
		StringBuffer search = new StringBuffer();
		search.append("(&(");
		search.append(OBJECTCLASS);
		search.append("=");
		search.append(OBJECT_CLASS_FACILITY);
		search.append(")(");
		search.append(OBJECTCLASS);
		search.append("=");
		search.append(OBJECT_CLASS_NODE);
		// IPアドレスを検索文字列に設定
		if(ipAddress != null && ipAddress.compareTo("") != 0){
			search.append(")(");
			search.append(FacilityAttributeConstant.IPNETWORKNUMBER);
			search.append("=");
			search.append(ipAddress);
		}
		search.append("))");
		
		//検索実行
		javax.naming.NamingEnumeration results = null;
		javax.naming.directory.DirContext ctx = null;
		int retryCount=0;
		
		while(MAX_RETRY > retryCount){
			try {
				// LDAP用のコンテキストの作成
				ctx = LdapConnectionManager.getConnectionManager().getDirContext();
				
				results = ctx.search(BASE_DN, search.toString(), constraints);
				
				if (results == null ) {
						m_log.warn("findByIpAddress() : No Search Result : ipAddress = " + ipAddress);
					return ret;
				}
	
				//検索結果を取得
				while ( results.hasMoreElements() ) {
					javax.naming.directory.SearchResult aResult = 
						(javax.naming.directory.SearchResult)results.nextElement();
//					FacilityPK pk = new FacilityPK(aResult.getName() + "," + BASE_DN);
					Attributes attrs = aResult.getAttributes();
					FacilityPK pk = new FacilityPK((String)attrs.get(FacilityAttributeConstant.FACILITYID).get(0));
					
					ret.add(pk);
				}
				
			} catch (NamingException ex) {
				if(ex instanceof CommunicationException){
		        LdapConnectionManager.getConnectionManager().setDirContext(null);
		    }
				m_log.debug("findByIpAddress() : " + ex.getMessage());
				
				//中身を空にします。
				ret.clear();
				
				// whileループの判定の前に、メッソッドを抜けるかのチェックをするため、
				// ここでカウントアップする。
				retryCount++;
				if(retryCount >= MAX_RETRY){
					throw new EJBException(ex);
				} else {
					//リトライ上限以下の場合には再度LDAPに問い合わせをします。
					//whlie loopに戻ります。
					continue;
				}
					
			} catch (NullPointerException e) {
				//jreのバグ（？）により、hasMoreElements()でNullPointerExceptionが発生することがある。
				//根本対処の方法がないため、NullPointerExcetionをcatchする。（2007/06/19　宮本）
				m_log.debug("findByIpAddress() : LDAP query result is null ");
				
				//中身を空にします。
				ret.clear();
				//whileループの判定の前に、メッソッドを抜けるかのチェックをするため、
				// ここでカウントアップする。
				retryCount++;
				if(retryCount >= MAX_RETRY){
					throw new EJBException(e);
				} else {
					//リトライ上限以下の場合には再度LDAPに問い合わせをします。
					//whlie loopに戻ります。
					continue;
				}
				
			} finally {
				try {
					if(results != null)
						results.close();
					if(ctx != null)
						ctx.close();
				} catch (NamingException e) {
					m_log.error("findByIpAddress() context close failed : " + e.getMessage());
				}
			}
	
			return ret;
		}
		return ret;
	}
	
    /* (non-Javadoc)
     * @see com.clustercontrol.repository.dao.FacilityDAO#findByNodeName(java.lang.String)
     */
    public Collection findByNodeName(String nodeName) throws FinderException {
		ArrayList ret = new ArrayList();
		
		m_log.debug("findByNodeName() : nodeName = " + nodeName);
		
		// エントリの検索
		javax.naming.directory.SearchControls constraints = 
			new javax.naming.directory.SearchControls();

		// BaseDN以下にぶら下がるすべてのレベルを検索の対象
		constraints.setSearchScope(javax.naming.directory.SearchControls.SUBTREE_SCOPE);

		// 検索文字列作成
		StringBuffer search = new StringBuffer();
		search.append("(&(");
		search.append(OBJECTCLASS);
		search.append("=");
		search.append(OBJECT_CLASS_FACILITY);
		search.append(")(");
		search.append(OBJECTCLASS);
		search.append("=");
		search.append(OBJECT_CLASS_NODE);
		// ノード名を検索文字列に設定
		if(nodeName != null && nodeName.compareTo("") != 0){
			search.append(")(");
			search.append(FacilityAttributeConstant.NODENAME);
			search.append("=");
			search.append(nodeName);
		}
		search.append("))");
		
		//検索実行
		javax.naming.NamingEnumeration results = null;
		javax.naming.directory.DirContext ctx = null;
		
		int retryCount=0;
		while(MAX_RETRY > retryCount){
			
				
			try {
				// LDAP用のコンテキストの作成
				ctx = LdapConnectionManager.getConnectionManager().getDirContext();
				
				results = ctx.search(BASE_DN, search.toString(), constraints);
				
				if (results == null ) {
					m_log.warn("findByNodeName() : No Search Result : nodeName = " + nodeName);
					return ret;
				}
	
				//検索結果を取得
				while ( results.hasMoreElements() ) {
					javax.naming.directory.SearchResult aResult = 
						(javax.naming.directory.SearchResult)results.nextElement();
//					FacilityPK pk = new FacilityPK(aResult.getName() + "," + BASE_DN);
					Attributes attrs = aResult.getAttributes();
					FacilityPK pk = new FacilityPK((String)attrs.get(FacilityAttributeConstant.FACILITYID).get(0));

					ret.add(pk);
				}
				
			} catch (NamingException ex) {
				if(ex instanceof CommunicationException){
					LdapConnectionManager.getConnectionManager().setDirContext(null);
				}
				m_log.debug("findByNodeName() : " + ex.getMessage());
				//中身を空にします。
				ret.clear();
				// whileループの判定の前に、メッソッドを抜けるかのチェックをするため、
				// ここでカウントアップする。
				retryCount++;
				if(retryCount >= MAX_RETRY){
					throw new EJBException(ex);
				} else {
					//リトライ上限以下の場合には再度LDAPに問い合わせをします。
					//whlie loopに戻ります。
					continue;
				}
					
			} catch (NullPointerException e) {
				//jreのバグ（？）により、hasMoreElements()でNullPointerExceptionが発生することがある。
				//根本対処の方法がないため、NullPointerExcetionをcatchする。（2007/06/19　宮本）
				m_log.debug("findByNodeName() : LDAP query result is null");
				//中身を空にします。
				ret.clear();
				//whileループの判定の前に、メッソッドを抜けるかのチェックをするため、
				// ここでカウントアップする。
				retryCount++;
				if(retryCount >= MAX_RETRY){
					throw new EJBException(e);
				} else {
					//リトライ上限以下の場合には再度LDAPに問い合わせをします。
					//whlie loopに戻ります。
					continue;
				}
				
			} finally {
				try {
					if(results != null)
						results.close();
					if(ctx != null)
						ctx.close();
				} catch (NamingException e) {
					m_log.error("findByNodeName() context close failed : " + e.getMessage());
				}
			}
			return ret;
	    }
		return ret;
	}
	/**
	 * StringをBooleanに変換する
	 * 
	 * @version 1.0.0
	 * @since 1.0.0
	 * 
	 * @param value
	 * @return
	 */
	public Boolean stringToBoolean(String value) {
		Boolean ret = null;
		
		if(value.compareTo("TRUE") == 0){
			ret = new Boolean(true);
		}
		else{
			ret = new Boolean(false);;
		}
		
		return ret;
	}
	
	/**
	 * BooleanをStringに変換する
	 * 
	 * @version 1.0.0
	 * @since 1.0.0
	 * 
	 * @param value
	 * @return
	 */
	public String booleanToString(Boolean value) {
		String ret = null;
		
		if(value.booleanValue()){
			ret = "TRUE";
		}
		else{
			ret = "FALSE";
		}
		
		return ret;
	}
	
	/**
	 * StringをDateに変換する
	 * 
	 * @version 1.0.0
	 * @since 1.0.0
	 * 
	 * @param value
	 * @return
	 */
	public Date stringToDate(String value) {
		Date ret = null;
		
		SimpleDateFormat formatter = new SimpleDateFormat("yyyyMMddHHmmss.SSS");

		try {
			ret = formatter.parse(value);
		} catch (ParseException e) {
		}

		return ret;
	}
	
	/**
	 * DateをStringに変換する
	 * 
	 * @version 1.0.0
	 * @since 1.0.0
	 * 
	 * @param value
	 * @return
	 */
	public String dateToString(Date value) {
		String ret = null;
		
		SimpleDateFormat formatter = new SimpleDateFormat("yyyyMMddHHmmss.SSSZ");

		ret = formatter.format(value);


		return ret;
	}
	
	/**
	 * Beanクリア処理
	 * 
	 * @version 1.0.0
	 * @since 1.0.0
	 * 
	 * @param ejb
	 */
	protected void clearBean(FacilityBean ejb){
	    ejb.setAgentControl(null);
	    ejb.setAgentWatch(null);
	    ejb.setAgentWatchStatus(null);
	    ejb.setCharSet(null);
	    ejb.setCn(null);
	    ejb.setCreateTimestamp(null);
	    ejb.setCreatorsName(null);
	    ejb.setDescription(null);
	    ejb.setDhcpClient(null);
	    ejb.setDn(null);
	    ejb.setFacilityId(null);
	    ejb.setHost(null);
	    ejb.setIpNetworkNumber(null);
	    ejb.setIpNetworkNumberV6(null);
	    ejb.setIpProtocolNumber(null);
	    ejb.setIpType(null);
	    ejb.setLogWatch(null);
	    ejb.setMachine(null);
	    ejb.setManagerContact(null);
	    ejb.setManagerName(null);
	    ejb.setModifiersName(null);
	    ejb.setModifyTimestamp(null);
	    ejb.setNodeName(null);
	    ejb.setOsName(null);
	    ejb.setOsRelease(null);
	    ejb.setOsVersion(null);
	    ejb.setPingWatch(null);
	    ejb.setPingWatchStatus(null);
	    ejb.setPlatform(null);
	    ejb.setStatusWatch(null);
	    ejb.setSnmpPort(null);
	    ejb.setSnmpCommunity(null);
	    ejb.setSnmpVersion(null);
	    ejb.setSortValue(null);
	}
	
	/**
	 * 
	 * facilityIdからdnの文字列を作成するためのプライベートメソッド
	 * 
	 * @param facilityId
	 * @return
	 */
	private String createDnString(String facilityId) {
		return "ccFacilityId=" + facilityId + "," + BASE_DN;
	}
}
